//
// here are three axis-drawing functions, one for X-axis,
// one for Y-axis and a third for both axes.  also
// included are two tic-drawing functions.
//

#include "gogo.h"

//
// draws an X axis, with origin at (x, y) (user units).  ticsep
// is separation between tic marks.  uptic and downtic should be
// set to a percentage of the window diagonal.  use 1.0 or so for
// normal tic marks.
//

void X_AXIS(flt x, flt y, flt ticsep, flt uptic, flt downtic)
{
    flt s, z = 0.0;
    flt xll, xur;
    int dyu, dyd;                      // uptic, downtic

    dyu = dyd = 0;
    M(x, y);                           // move to starting point
    if (uptic != 0.0){
        z = windiag();                 // z is window diagonal
        dyu = (int)(0.01 * uptic * z); // dyu is uptic % of z
    }
    if (downtic != 0.0){
        if (z != 0.0){                 // dyd is downtic % of z
            dyd = (int)(0.01 * downtic * z);
        }
        else {
            z = windiag();             // z is window diagonal
            dyd = (int)(0.01 * downtic * z);
        }
    }
    xll = (w[0] - 0.5 - trm[0][2])/trm[0][0]; // < left window edge
    xur = (w[2] + 0.5 - trm[0][2])/trm[0][0]; // > right window edge
    if (dyd || dyu){                   // draw tics if they exist
        for (s = x; s < xur ; s += ticsep){
            L(s, y);
            draw_ytic(dyu, dyd);
        }
    }
    L(xur, y);                         // extend axis to window edge
    M(x, y);                           // move to starting point
    if (dyd || dyu){                   // draw tics if they exist
        for (s = x - ticsep; s > xll ; s -= ticsep){
            L(s, y);
            draw_ytic(dyu, dyd);
        }
    }
    L(xll, y);      // extend axis to window edge
}
// =========================================================
//
// draws a Y axis, with origin at (x, y) (user units).  ticsep
// is separation between tic marks.  righttic and lefttic should be
// set to a percentage of the window diagonal.  use 1.0 or so for
// normal tic marks.
//

void Y_AXIS(flt x, flt y, flt ticsep, flt righttic, flt lefttic)
{
    flt s, z = 0.0;
    flt yll, yur; 
    int dxr, dxl;

    dxr = dxl = 0;
    M(x, y);                              // move to starting point
    if (righttic != 0.0){
        z  = windiag();                   // z is window diagonal
        dxr = (int)(0.01 * righttic * z); // dxr is righttic % of z
    }
    if (lefttic != 0.0){
        if (z != 0.0){                    // dxl is lefttic % of z
            dxl = (int)(0.01 * lefttic * z);
        }
        else{
            z  = windiag();               // z is window diagonal
            dxl = (int)(0.01 * lefttic * z);
        }
    }
    yll = (w[1] - 0.5 - trm[1][2])/trm[1][1]; // < bottom window edge
    yur = (w[3] + 0.5 - trm[1][2])/trm[1][1]; // > top window edge
    if (dxr || dxl){              // draw tics if they exist     
        for (s = y; s < yur ; s += ticsep){
            L(x, s);
            draw_xtic(dxr, dxl);  // draw a tic mark
        }
    }
    L(x, yur);                    // extend axis to window edge
    M(x, y);                      // move to starting point
    if (dxr || dxl){              // draw tics if they exist     
        for (s = y - ticsep; s > yll ; s -= ticsep){
            L(x, s);
            draw_xtic(dxr, dxl);  // draw a tic mark
        }
    }
    L(x, yll);                    // extend axis to window edge
}
// =========================================================
//
// draws a pair of crossed axes through the point (x, y).
//
// xtic is tic separation on x-axis;
// ytic is tic separation on y-axis.
//
// x, y, xtic, ytic all in user units.
//
// ur_tic is tic length for up  and right tics;
// dl_tic is tic length for down and left tics.
//
// tic lengths are in per cent of window diagonal.
//

void AXES(flt x, flt y, flt xtic, flt ytic,\
           flt ur_tic, flt dl_tic)
{
    X_AXIS(x, y, xtic, ur_tic, dl_tic);
    Y_AXIS(x, y, ytic, ur_tic, dl_tic);
}
// =========================================================
//
// draws a horizontal tic of length dxr and dxl on y-axis
//

void draw_xtic(int dxr, int dxl)
{
    if (dxr){
        RLG(dxr, 0);  // draws tic to right
        RMG(-dxr, 0); // returns to y axis
    }
    if (dxl){
        RLG(-dxl, 0); // draws tic to left
        RMG(dxl, 0);  // returns to y axis
    }
}
// =========================================================
//
// draws a vertical tic of length dyu and dyd on x-axis
//

void draw_ytic(int dyu, int dyd)
{
    if (dyu){
        RLG(0,  dyu); // draws tic up
        RMG(0, -dyu); // returns to axis
    }
    if (dyd){
        RLG(0, -dyd); // draws tic down
        RMG(0,  dyd); // returns to axis
    }
}
// =========================================================
