// gogo.h 1/26/97 version 1.1
// gogo.h 2/22/06 version 1.2

#include <cstdlib>
#include <cstdio>
#include <string>
#include <cmath>
#include <cstdarg>
#include <iostream>
#include <vector>
#include <termios.h>
#include <unistd.h>
#include "util.h"
#include "errors.h"

typedef float                     flt;
typedef double                    dbl;
typedef const float              cflt;
typedef unsigned                  uns;
typedef const unsigned           cuns;
typedef const int                cint;
typedef vector<vector<flt> >  fmatrix; // for trm matrix
typedef struct termios            tio; // for hotkeys

enum off_on    {off, on};
enum no_yes    {no, yes};
enum term_file {term, file}; // select output venue

cint URX_MAX = 1023;     // upper right xmax
cint URY_MAX = 781;      // upper right ymax
cflt PTS_PER_IN = 100.0; // 10.23 in by 7.81 in
cuns MAXLINE = 200;

// #define BSLASH '\\'

void CLEAR(void);     // clear graphics screen
void INITGRAPH(void); // initialize graphics
void out(ofstream&, string); // select output venue
void out(ofstream&, flt);    // select output venue
void out(ofstream&, char, char, char, char); // select output venue
void out(ofstream&, char, char, char, char, char); // select output venue
void M(flt, flt); // move to usr x, usr y
void L(flt, flt); // draw line to usr x, usr y
void MG(vector<int>&); // move to graph g[0], graph g[1]
void MG(int, int);     // move to graph xg, graph yg
void LG(vector<int>&); // draw line to graph g[0], graph g[1]
void LG(int, int);     // draw line to graph xg, graph yg
void RLG(int, int);    // draw line rel to glast[] by (rxg, ryg)
void RMG(int, int);    // move rel to glast[] by (rxg, ryg)
void DOT(flt, flt);      // puts a dot at usr x, usr y
void BOX(flt, flt, flt); // draws box at usr x, usr y
// void TRIANGLE( float, float, float) ;// triangle at usr x, usr y
// void DEL( float, float, float) ;     // draws "del" at usr x, usr y
// void CIRCLE( float, float, float ) ; // draws circle at usr x, usr y
// void ERRBAR( float, float, float ) ; // draws err bar at usr x, usr y
// void ARROW( float, float, float, float ) ; // arrow point at usr x, y
void STYLE(uns);      // selects alternate line patterns
void GWIN(void);      // switch to graphics window
void AWIN(void);      // switch to alphanumeric window
void GMODE(off_on);   // set graph mode to plot (on) or text (off)
void END(void);       // put at end of a graphics program
// void BLKFIL(int);     // turn block fill on or off
void COLOR(int);      // sets data on (1), off (2), inverted (3)
void TEXT(uns, string);    // writes text on graph window
void TEXT(uns, string, flt);    // writes text on graph window
vector<string> form_string(const string); // for use in TEXT functions
void newline(uns);
void WINDOW(flt, flt, flt, flt);
void SCALE(flt, flt, flt, flt);
void BORDER(void);    // draws a border at window boundaries
void X_AXIS(flt, flt, flt, flt, flt);
void Y_AXIS(flt, flt, flt, flt, flt);
void AXES(flt, flt, flt, flt, flt, flt);
vector<int> GIN();      // enters GIN mode, returns g vector
vector<int> GETXY();    // returns g vector of point
void hotkeys(off_on);       // turns hot keyboard on or off

flt windiag(void);          // calculates window diagonal
void moveto(vector<int>&);  // unprotected move to (xg, yg)
void lineto(vector<int>&);  // unprotected line to (xg, yg)
vector<int> u2g(vector<flt>&); // transforms u to g
vector<flt> g2u(vector<int>&); // transforms g to u
void draw_ytic(int, int);   // draws a tic mark in y direction
void draw_xtic(int, int);   // draws a tic mark in x direction
// void convert( char *, char * ) ; // converts \dd to ascii chars in TEXT

// the following variables are global, defined in
// defs.c

extern uns gwin;          // gwin = true if in graphics window
extern uns gmode;         // gmode = true if in graphics mode
extern uns xterm;         // xterm = true if xterm
extern uns oob;           // oob = true if out of bounds
extern vector<int> w;     // vector of window corners
extern fmatrix trm;       // transformation matrix
extern vector<int> glast; // vector of previous xg, yg
extern term_file oflg;    // term: to terminal; file: to file
                          // used with out()
extern ofstream ofs;      // for printing to a file
